@@ -90,7 +90,7 @@ Arith(void *process)
     FILL_VARTS,
     FILL_FILE
   };
-  int filltype = FILL_NONE;
+  int fillType = FILL_NONE;
   int nlevels2 = 1;
   int levelID2 = -1;
   Varray2D<size_t> varnmiss;
@@ -114,8 +114,8 @@ Arith(void *process)
   auto streamID2x = streamID2;
 
   Field field1, field2;
-  Field *fieldx1 = &field1;
-  Field *fieldx2 = &field2;
+  auto fieldx1 = &field1;
+  auto fieldx2 = &field2;
 
   auto vlistID1 = cdo_stream_inq_vlist(streamID1);
   auto vlistID2 = cdo_stream_inq_vlist(streamID2);
@@ -136,58 +136,58 @@ Arith(void *process)
   const auto nvars1 = vlistNvars(vlistID1);
   const auto nvars2 = vlistNvars(vlistID2);
 
-  auto lfill1 = false, lfill2 = false, lfill1x = false;
+  auto fillStream1 = false, fillStream2 = false, fillStream1x = false;
   if (nvars1 == 1 && nvars2 == 1)
     {
-      lfill2 = vlistNrecs(vlistID1) != 1 && vlistNrecs(vlistID2) == 1;
-      lfill1 = vlistNrecs(vlistID1) == 1 && vlistNrecs(vlistID2) != 1;
-      if (lfill1 && ntsteps1 != 1 && ntsteps2 == 1)
+      fillStream2 = (vlistNrecs(vlistID1) != 1 && vlistNrecs(vlistID2) == 1);
+      fillStream1 = (vlistNrecs(vlistID1) == 1 && vlistNrecs(vlistID2) != 1);
+      if (fillStream1 && ntsteps1 != 1 && ntsteps2 == 1)
         {
-          lfill1 = false;
-          lfill2 = true;
-          lfill1x = true;
+          fillStream1 = false;
+          fillStream2 = true;
+          fillStream1x = true;
         }
     }
   else
     {
-      lfill2 = nvars1 != 1 && nvars2 == 1;
-      lfill1 = nvars1 == 1 && nvars2 != 1;
+      fillStream2 = (nvars1 != 1 && nvars2 == 1);
+      fillStream1 = (nvars1 == 1 && nvars2 != 1);
     }
 
-  if (lfill1x)
+  if (fillStream1x)
     {
       nlevels2 = vlist_compare_x(vlistID2, vlistID1, CMP_DIM);
 
-      filltype = FILL_NONE;
+      fillType = FILL_NONE;
       cdo_print("Filling up stream1 >%s< by copying the first variable of each timestep.", cdo_get_stream_name(0));
     }
-  else if (lfill2)
+  else if (fillStream2)
     {
       nlevels2 = vlist_compare_x(vlistID1, vlistID2, CMP_DIM);
 
       if (ntsteps1 != 1 && ntsteps2 == 1)
         {
-          filltype = FILL_VAR;
+          fillType = FILL_VAR;
           cdo_print("Filling up stream2 >%s< by copying the first variable.", cdo_get_stream_name(1));
         }
       else
         {
-          filltype = FILL_VARTS;
+          fillType = FILL_VARTS;
           cdo_print("Filling up stream2 >%s< by copying the first variable of each timestep.", cdo_get_stream_name(1));
         }
     }
-  else if (lfill1)
+  else if (fillStream1)
     {
       nlevels2 = vlist_compare_x(vlistID2, vlistID1, CMP_DIM);
 
       if (ntsteps1 == 1 && ntsteps2 != 1)
         {
-          filltype = FILL_VAR;
+          fillType = FILL_VAR;
           cdo_print("Filling up stream1 >%s< by copying the first variable.", cdo_get_stream_name(0));
         }
       else
         {
-          filltype = FILL_VARTS;
+          fillType = FILL_VARTS;
           cdo_print("Filling up stream1 >%s< by copying the first variable of each timestep.", cdo_get_stream_name(0));
         }
 
@@ -199,7 +199,7 @@ Arith(void *process)
       fieldx2 = &field1;
     }
 
-  if (lfill1x == false && filltype == FILL_NONE) vlist_compare(vlistID1, vlistID2, CMP_ALL);
+  if (fillStream1x == false && fillType == FILL_NONE) vlist_compare(vlistID1, vlistID2, CMP_ALL);
 
   const size_t nwpv = (vlistNumber(vlistID1x) == CDI_COMP && vlistNumber(vlistID2x) == CDI_COMP) ? 2 : 1;
   if (nwpv == 2 && !opercplx) cdo_abort("Fields with complex numbers are not supported by this operator!");
@@ -208,7 +208,7 @@ Arith(void *process)
   field1.resize(gridsizemax);
   field2.resize(gridsizemax);
 
-  if (lfill1x || filltype == FILL_VAR || filltype == FILL_VARTS)
+  if (fillStream1x || fillType == FILL_VAR || fillType == FILL_VARTS)
     {
       vardata2.resize(gridsizemax * nlevels2);
       varnmiss2.resize(nlevels2);
@@ -216,16 +216,16 @@ Arith(void *process)
 
   if (Options::cdoVerbose) cdo_print("Number of timesteps: file1 %d, file2 %d", ntsteps1, ntsteps2);
 
-  if (filltype == FILL_NONE)
+  if (fillType == FILL_NONE)
     {
       if (ntsteps1 != 1 && ntsteps2 == 1)
         {
-          filltype = FILL_TS;
+          fillType = FILL_TS;
           cdo_print("Filling up stream2 >%s< by copying the first timestep.", cdo_get_stream_name(1));
         }
       else if (ntsteps1 == 1 && ntsteps2 != 1)
         {
-          filltype = FILL_TS;
+          fillType = FILL_TS;
           cdo_print("Filling up stream1 >%s< by copying the first timestep.", cdo_get_stream_name(0));
 
           std::swap(streamID1, streamID2);
@@ -236,10 +236,10 @@ Arith(void *process)
           fieldx2 = &field1;
         }
 
-      if (filltype == FILL_TS) cdo_fill_ts(vlistID2x, vardata, varnmiss);
+      if (fillType == FILL_TS) cdo_fill_ts(vlistID2x, vardata, varnmiss);
     }
 
-  const auto streamsSwaped = (filltype == FILL_TS && vlistID1x != vlistID1);
+  const auto streamsSwaped = (fillType == FILL_TS && vlistID1x != vlistID1);
 
   const auto vlistID3 = vlistDuplicate(streamsSwaped ? vlistID1 : vlistID1x);
   if (streamsSwaped)
@@ -249,7 +249,7 @@ Arith(void *process)
       for (int varID = 0; varID < nvars; varID++) vlistDefVarTimetype(vlistID3, varID, vlistInqVarTimetype(vlistID1x, varID));
     }
 
-  if (lfill1x)
+  if (fillStream1x)
     {
       const auto zaxisID2 = vlistZaxis(vlistID2x, 0);
       vlistChangeZaxisIndex(vlistID3, 0, zaxisID2);
@@ -273,20 +273,20 @@ Arith(void *process)
       nrecs = cdo_stream_inq_timestep(streamID1, tsID);
 
       nrecs2 = 0;
-      if (tsID == 0 || filltype == FILL_NONE || filltype == FILL_FILE || filltype == FILL_VARTS)
+      if (tsID == 0 || fillType == FILL_NONE || fillType == FILL_FILE || fillType == FILL_VARTS)
         {
           nrecs2 = cdo_stream_inq_timestep(streamID2, tsID2);
           if (nrecs2 == 0)
             {
               if (nrecs == 0) break;
 
-              if (filltype == FILL_NONE && streamID2x == streamID2)
+              if (fillType == FILL_NONE && streamID2x == streamID2)
                 {
-                  filltype = FILL_FILE;
+                  fillType = FILL_FILE;
                   cdo_print("Filling up stream2 >%s< by copying all timesteps.", cdo_get_stream_name(1));
                 }
 
-              if (filltype == FILL_FILE)
+              if (fillType == FILL_FILE)
                 {
                   cdo_stream_close(streamID2);
 
@@ -308,23 +308,23 @@ Arith(void *process)
             }
         }
 
-      if (nrecs == 0 || (nrecs2 == 0 && filltype != FILL_TS)) break;
+      if (nrecs == 0 || (nrecs2 == 0 && fillType != FILL_TS && fillType != FILL_VAR)) break;
 
       cdo_taxis_copy_timestep(taxisID3, taxisID1);
       cdo_def_timestep(streamID3, tsID);
 
-      const auto numrecs = lfill1x ? nrecs2 : nrecs;
+      const auto numrecs = fillStream1x ? nrecs2 : nrecs;
       for (int recID = 0; recID < numrecs; recID++)
         {
           auto lread1 = true;
-          if (lfill1x && recID > 0) lread1 = false;
+          if (fillStream1x && recID > 0) lread1 = false;
           int varID = -1, levelID;
           if (lread1)
             {
               cdo_inq_record(streamID1, &varID, &levelID);
               cdo_read_record(streamID1, fieldx1->vec_d.data(), &fieldx1->nmiss);
 
-              if (lfill1x)
+              if (fillStream1x)
                 {
                   const auto gridsize = nwpv * varList1[varID].gridsize;
                   array_copy(gridsize, fieldx1->vec_d.data(), &vardata2[0]);
@@ -332,29 +332,29 @@ Arith(void *process)
                 }
             }
 
-          if (lfill1x) levelID = recID;
+          if (fillStream1x) levelID = recID;
 
           auto varID2 = varID;
 
-          if (tsID == 0 || filltype == FILL_NONE || filltype == FILL_FILE || filltype == FILL_VARTS)
+          if (tsID == 0 || fillType == FILL_NONE || fillType == FILL_FILE || fillType == FILL_VARTS)
             {
               const auto lstatus = (nlevels2 > 1) ? varID == 0 : recID == 0;
-              if (lstatus || (filltype != FILL_VAR && filltype != FILL_VARTS))
+              if (lstatus || (fillType != FILL_VAR && fillType != FILL_VARTS))
                 {
                   cdo_inq_record(streamID2, &varID2, &levelID2);
                   cdo_read_record(streamID2, fieldx2->vec_d.data(), &fieldx2->nmiss);
                   if (varID != varID2) cdo_abort("Internal error, varIDs of input streams differ!");
-                  if (lfill1x == false && levelID != levelID2) cdo_abort("Internal error, levelIDs of input streams differ!");
+                  if (fillStream1x == false && levelID != levelID2) cdo_abort("Internal error, levelIDs of input streams differ!");
                 }
 
-              if (filltype == FILL_TS)
+              if (fillType == FILL_TS)
                 {
                   const auto gridsize = nwpv * varList2[varID].gridsize;
                   const auto offset = gridsize * levelID;
                   array_copy(gridsize, fieldx2->vec_d.data(), &vardata[varID][offset]);
                   varnmiss[varID][levelID] = fieldx2->nmiss;
                 }
-              else if (lstatus && (filltype == FILL_VAR || filltype == FILL_VARTS))
+              else if (lstatus && (fillType == FILL_VAR || fillType == FILL_VARTS))
                 {
                   const auto gridsize = nwpv * varList2[0].gridsize;
                   const auto offset = gridsize * levelID2;
@@ -362,7 +362,7 @@ Arith(void *process)
                   varnmiss2[levelID2] = fieldx2->nmiss;
                 }
             }
-          else if (filltype == FILL_TS)
+          else if (fillType == FILL_TS)
             {
               const auto gridsize = nwpv * varList2[varID2].gridsize;
               const auto offset = gridsize * levelID;
@@ -370,7 +370,7 @@ Arith(void *process)
               fieldx2->nmiss = varnmiss[varID][levelID];
             }
 
-          if (lfill1x)
+          if (fillStream1x)
             {
               const auto gridsize = nwpv * varList1[0].gridsize;
               array_copy(gridsize, &vardata2[0], fieldx1->vec_d.data());
@@ -381,7 +381,7 @@ Arith(void *process)
           fieldx1->missval = varList1[varID].missval;
           fieldx1->nwpv = varList1[varID].nwpv;
 
-          if (filltype == FILL_VAR || filltype == FILL_VARTS)
+          if (fillType == FILL_VAR || fillType == FILL_VARTS)
             {
               levelID2 = (nlevels2 > 1) ? levelID : 0;
               const auto gridsize = nwpv * varList2[0].gridsize;
