@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -32,7 +22,7 @@
 
 #include <cdi.h>
 
-#include "array.h"
+#include "varray.h"
 #include "cdo_options.h"
 #include "process_int.h"
 #include "param_conversion.h"
@@ -46,8 +36,8 @@ static int
 check_ncorner(int ncorner, const double *lon_bounds, const double *lat_bounds)
 {
   auto ncorner_new = ncorner;
-  int k;
 
+  int k;
   for (k = ncorner - 1; k > 0; --k)
     if (IS_NOT_EQUAL(lon_bounds[k], lon_bounds[k - 1]) || IS_NOT_EQUAL(lat_bounds[k], lat_bounds[k - 1])) break;
 
@@ -56,20 +46,40 @@ check_ncorner(int ncorner, const double *lon_bounds, const double *lat_bounds)
   return ncorner_new;
 }
 
+static void
+check_lonbounds(int ncorner, double *lon_bounds)
+{
+  auto isLtM90 = false;
+  auto isGtP90 = false;
+  for (int k = 0; k < ncorner; ++k)
+    {
+      if (lon_bounds[k] < -90.0) isLtM90 = true;
+      if (lon_bounds[k] >  90.0) isGtP90 = true;
+    }
+
+  if (isLtM90 && isGtP90)
+    {
+      for (int k = 0; k < ncorner; ++k)
+        if (lon_bounds[k] < -90.0) lon_bounds[k] += 360.0;
+   
+      // printf("XXXX %g %g %g\n", lon_bounds[0], lon_bounds[1], lon_bounds[2]);
+    }
+}
+
 static void
 make_cyclic(double *array1, double *array2, long nlon, long nlat)
 {
   for (long j = 0; j < nlat; ++j)
     for (long i = 0; i < nlon; ++i)
       {
-        const long ij1 = j * nlon + i;
-        const long ij2 = j * (nlon + 1) + i;
+        const auto ij1 = j * nlon + i;
+        const auto ij2 = j * (nlon + 1) + i;
         array2[ij2] = array1[ij1];
       }
 
   for (long j = 0; j < nlat; ++j)
     {
-      const long ij2 = j * (nlon + 1);
+      const auto ij2 = j * (nlon + 1);
       array2[ij2 + nlon] = array2[ij2];
     }
 }
@@ -77,19 +87,12 @@ make_cyclic(double *array1, double *array2, long nlon, long nlat)
 static void
 output_zon(double levmin, double levmax, const double *cell_corner_lat)
 {
-  double xlev[4], xlat[4];
   auto latmin = cell_corner_lat[0];
   auto latmax = cell_corner_lat[0];
   for (int ic = 1; ic < 4; ic++) latmin = std::min(latmin, cell_corner_lat[ic]);
   for (int ic = 1; ic < 4; ic++) latmax = std::max(latmax, cell_corner_lat[ic]);
-  xlev[0] = levmin;
-  xlev[1] = levmax;
-  xlev[2] = levmax;
-  xlev[3] = levmin;
-  xlat[0] = latmin;
-  xlat[1] = latmin;
-  xlat[2] = latmax;
-  xlat[3] = latmax;
+  const double xlev[4] = { levmin, levmax, levmax, levmin };
+  const double xlat[4] = { latmin, latmin, latmax, latmax };
   for (int ic = 0; ic < 4; ic++) fprintf(stdout, "   %g  %g\n", xlat[ic], xlev[ic]);
   fprintf(stdout, "   %g  %g\n", xlat[0], xlev[0]);
 }
@@ -97,62 +100,38 @@ output_zon(double levmin, double levmax, const double *cell_corner_lat)
 static void
 output_mer(double levmin, double levmax, const double *cell_corner_lon)
 {
-  double xlev[4], xlon[4];
   auto lonmin = cell_corner_lon[0];
   auto lonmax = cell_corner_lon[0];
   for (int ic = 1; ic < 4; ic++) lonmin = std::min(lonmin, cell_corner_lon[ic]);
   for (int ic = 1; ic < 4; ic++) lonmax = std::max(lonmax, cell_corner_lon[ic]);
-  xlev[0] = levmin;
-  xlev[1] = levmin;
-  xlev[2] = levmax;
-  xlev[3] = levmax;
-  xlon[0] = lonmin;
-  xlon[1] = lonmax;
-  xlon[2] = lonmax;
-  xlon[3] = lonmin;
+  const double xlev[4] = { levmin, levmin, levmax, levmax };
+  const double xlon[4] = { lonmin, lonmax, lonmax, lonmin };
   for (int ic = 0; ic < 4; ic++) fprintf(stdout, "   %g  %g\n", xlon[ic], xlev[ic]);
   fprintf(stdout, "   %g  %g\n", xlon[0], xlev[0]);
 }
 
-static void
-output_boundscpt(double value, double missval, const CPT &cpt)
+static const int*
+get_rgb(double value, double missval, const CPT &cpt)
 {
-  int r = 0, g = 0, b = 0, n;
-
   if (!DBL_IS_EQUAL(value, missval))
     {
+      int n;
       for (n = 0; n < cpt.ncolors; n++)
         if (value > cpt.lut[n].z_low && value <= cpt.lut[n].z_high) break;
 
-      if (n == cpt.ncolors)
-        {
-          r = cpt.bfn[0].rgb[0];
-          g = cpt.bfn[0].rgb[1];
-          b = cpt.bfn[0].rgb[2];
-        }
-      else
-        {
-          r = cpt.lut[n].rgb_high[0];
-          g = cpt.lut[n].rgb_high[1];
-          b = cpt.lut[n].rgb_high[2];
-        }
+      return (n == cpt.ncolors) ? cpt.bfn[0].rgb : cpt.lut[n].rgb_high;
     }
   else
     {
-      r = cpt.bfn[2].rgb[0];
-      g = cpt.bfn[2].rgb[1];
-      b = cpt.bfn[2].rgb[2];
+      return cpt.bfn[2].rgb;
     }
-
-  fprintf(stdout, " -G%d/%d/%d", r, g, b);
 }
-                 
+
 static void
 output_vrml(long nlon, long nlat, long ngp, const Varray<double> &array, double missval, const CPT &cpt)
 {
-  const auto mm = varrayMinMaxMV(ngp, array, missval);
-
-  double dx = 10. / nlon;
+  const auto mm = varray_min_max_mv(ngp, array, missval);
+  const auto dx = 10.0 / nlon;
 
   printf("Viewpoint {\n");
   printf("  description \"viewpoint1\"\n");
@@ -243,7 +222,119 @@ output_vrml(long nlon, long nlat, long ngp, const Varray<double> &array, double
 }
 
 static void
-output_vector(long nlon, long nlat, int ninc, const Varray<double> &lon, const Varray<double> &lat, const Varray<double> &uf, const Varray<double> &vf)
+output_kml(size_t ngp, const Varray<double> &array, int ncorner, const Varray<double> &grid_corner_lat, Varray<double> &grid_corner_lon,
+           const std::vector<int> &grid_mask, double missval, const CPT &cpt)
+{
+  fprintf(stdout, "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
+  fprintf(stdout, "<kml xmlns=\"http://www.opengis.net/kml/2.2\" \n");
+  fprintf(stdout, "     xmlns:atom=\"http://www.w3.org/2005/Atom\">\n");
+  fprintf(stdout, "<Document>\n");
+  fprintf(stdout, " <name>CDO plot</name>\n");
+  fprintf(stdout, " <open>1</open>\n");
+#ifdef VERSION
+  fprintf(stdout, " <atom:generator>CDO %s</atom:generator>\n", VERSION);
+#endif
+  fprintf(stdout, " <description>\n");
+  fprintf(stdout, " <![CDATA[Generated by CDO]]>\n");
+  fprintf(stdout, " </description>\n");
+  fprintf(stdout, " <LookAt>\n");
+  fprintf(stdout, "	<longitude>0</longitude>\n");
+  fprintf(stdout, "	<latitude>40</latitude>\n");
+  fprintf(stdout, "	<range>6e+06</range>\n");
+  fprintf(stdout, "	<tilt>0</tilt>\n");
+  fprintf(stdout, "	<heading>0</heading>\n");
+  fprintf(stdout, "	<altitudeMode>absolute</altitudeMode>\n");
+  fprintf(stdout, " </LookAt>\n");
+  fprintf(stdout, " <Style id=\"check-hide-children\">\n");
+  fprintf(stdout, "  <ListStyle>\n");
+  fprintf(stdout, "   <listItemType>checkHideChildren</listItemType>\n");
+  fprintf(stdout, "  </ListStyle>\n");
+  fprintf(stdout, " </Style>\n");
+  fprintf(stdout, "<Folder>\n");
+  fprintf(stdout, "<name>Layer:Page</name>\n");
+  fprintf(stdout, "<open>0</open>\n");
+  fprintf(stdout, " <styleUrl>#check-hide-children</styleUrl>\n");
+  fprintf(stdout, "<TimeSpan>\n");
+  fprintf(stdout, " <begin></begin>\n");
+  fprintf(stdout, " <end></end>\n");
+  fprintf(stdout, "</TimeSpan>\n");
+  fprintf(stdout, "<description><![CDATA[Layer:Page]]></description>\n");
+  fprintf(stdout, "<Folder>\n");
+  fprintf(stdout, "<name>Layer:no_name</name>\n");
+  fprintf(stdout, "<open>0</open>\n");
+  fprintf(stdout, " <styleUrl>#check-hide-children</styleUrl>\n");
+  fprintf(stdout, "<TimeStamp>\n");
+  fprintf(stdout, " <when>2021-11-04T13:19:00Z</when>\n");
+  fprintf(stdout, "</TimeStamp>\n");
+  fprintf(stdout, "<styleUrl>#hiker-icon</styleUrl>\n");
+  fprintf(stdout, "<description><![CDATA[Layer:no_name]]></description>\n");
+
+  int height = 5000;
+  for (size_t i = 0; i < ngp; i++)
+    {
+      if (grid_mask.size() && grid_mask[i] == 0) continue;
+
+      const auto lonBounds = &grid_corner_lon[i * ncorner];
+      const auto latBounds = &grid_corner_lat[i * ncorner];
+      const auto ncornerNew = check_ncorner(ncorner, lonBounds, latBounds);
+      check_lonbounds(ncornerNew, lonBounds);
+
+      auto rgb = get_rgb(array[i], missval, cpt);
+
+      fprintf(stdout, "<Placemark>\n");
+      fprintf(stdout, "<visibility>1</visibility>\n");
+      fprintf(stdout, "<open>0</open>\n");
+      fprintf(stdout, "<Style>\n");
+      fprintf(stdout, "<PolyStyle>\n");
+      fprintf(stdout, "	<!-- r:%d g:%d b:%d -->\n", rgb[0], rgb[1], rgb[2]);
+      fprintf(stdout, "	<color>fe%x%x%x</color>\n", (unsigned char)rgb[0], (unsigned char)rgb[1], (unsigned char)rgb[2]);
+      fprintf(stdout, "	<fill>1</fill>\n");
+      fprintf(stdout, "</PolyStyle>\n");
+      fprintf(stdout, "<LineStyle>\n");
+      fprintf(stdout, "	<width>2</width>\n");
+      fprintf(stdout, "	<!-- r:%d g:%d b:%d -->\n", rgb[0], rgb[1], rgb[2]);
+      fprintf(stdout, "	<color>fe%x%x%x</color>\n", (unsigned char)rgb[0], (unsigned char)rgb[1], (unsigned char)rgb[2]);
+      fprintf(stdout, "</LineStyle>\n");
+      fprintf(stdout, "</Style>\n");
+      fprintf(stdout, "<MultiGeometry>\n");
+
+      fprintf(stdout, "<Polygon>\n");
+      fprintf(stdout, "<extrude>1</extrude>\n");
+      fprintf(stdout, "<altitudeMode>clampToGround</altitudeMode>\n");
+      fprintf(stdout, "<tessellate>0</tessellate>\n");
+      fprintf(stdout, " <outerBoundaryIs>\n");
+      fprintf(stdout, "  <LinearRing>\n");
+      fprintf(stdout, "   <coordinates>\n");
+
+      for (int ic = 0; ic < ncornerNew; ic++) fprintf(stdout, "     %g,%g,%d\n", lonBounds[ic], latBounds[ic], height);
+      fprintf(stdout, "     %g,%g,%d\n", lonBounds[0], latBounds[0], height);
+
+      fprintf(stdout, "   </coordinates>\n");
+      fprintf(stdout, "  </LinearRing>\n");
+      fprintf(stdout, " </outerBoundaryIs>\n");
+      fprintf(stdout, "</Polygon>\n");
+      fprintf(stdout, "</MultiGeometry>\n");
+      fprintf(stdout, "</Placemark>\n");
+    }
+
+  fprintf(stdout, "</Folder>\n");
+  fprintf(stdout, "<ScreenOverlay id=\"legend\">\n");
+  fprintf(stdout, "<name>Legend</name>\n");
+  fprintf(stdout, "<Icon>\n");
+  fprintf(stdout, " <href>legend.png</href>\n");
+  fprintf(stdout, "</Icon>\n");
+  fprintf(stdout, "<overlayXY x=\"0\" y=\"0\" xunits=\"fraction\" yunits=\"fraction\"/>\n");
+  fprintf(stdout, "<screenXY x=\"0\" y=\"0\" xunits=\"fraction\" yunits=\"fraction\"/>\n");
+  fprintf(stdout, "<size x=\"-1\" y=\"0.1\" xunits=\"fraction\" yunits=\"fraction\"/>\n");
+  fprintf(stdout, "</ScreenOverlay>\n");
+  fprintf(stdout, "</Folder>\n");
+  fprintf(stdout, "</Document>\n");
+  fprintf(stdout, "</kml>\n");
+}
+
+static void
+output_vector(long nlon, long nlat, int ninc, const Varray<double> &lon, const Varray<double> &lat, const Varray<double> &uf,
+              const Varray<double> &vf)
 {
   for (long j = 0; j < nlat; j += ninc)
     for (long i = 0; i < nlon; i += ninc)
@@ -273,55 +364,57 @@ Outputgmt(void *process)
   size_t nmiss;
   int ninc = 1;
   bool lzon = false, lmer = false, lhov = false;
-  bool luse_grid_corner = false;
   char varname[CDI_MAX_NAME];
   Varray<double> grid_center_lat2, grid_center_lon2;
   Varray<double> grid_corner_lat, grid_corner_lon;
   std::vector<int> grid_mask;
   CPT cpt;
 
-  cdoInitialize(process);
+  cdo_initialize(process);
 
   // clang-format off
-  const auto OUTPUTCENTER    = cdoOperatorAdd("gmtxyz",          0, 0, nullptr);
-  const auto OUTPUTCENTER2   = cdoOperatorAdd("outputcenter2",   0, 0, nullptr);
-  const auto OUTPUTCENTERCPT = cdoOperatorAdd("outputcentercpt", 0, 0, nullptr);
-  const auto OUTPUTBOUNDS    = cdoOperatorAdd("gmtcells",        0, 0, nullptr);
-  const auto OUTPUTBOUNDSCPT = cdoOperatorAdd("outputboundscpt", 0, 0, nullptr);
-  const auto OUTPUTVECTOR    = cdoOperatorAdd("outputvector",    0, 0, nullptr);
-  const auto OUTPUTTRI       = cdoOperatorAdd("outputtri",       0, 0, nullptr);
-  const auto OUTPUTVRML      = cdoOperatorAdd("outputvrml",      0, 0, nullptr);
+  const auto OUTPUTCENTER    = cdo_operator_add("gmtxyz",          0, 0, nullptr);
+  const auto OUTPUTCENTER2   = cdo_operator_add("outputcenter2",   0, 0, nullptr);
+  const auto OUTPUTCENTERCPT = cdo_operator_add("outputcentercpt", 0, 0, nullptr);
+  const auto OUTPUTBOUNDS    = cdo_operator_add("gmtcells",        0, 0, nullptr);
+  const auto OUTPUTBOUNDSCPT = cdo_operator_add("outputboundscpt", 0, 0, nullptr);
+  const auto OUTPUTVECTOR    = cdo_operator_add("outputvector",    0, 0, nullptr);
+  const auto OUTPUTTRI       = cdo_operator_add("outputtri",       0, 0, nullptr);
+  const auto OUTPUTVRML      = cdo_operator_add("outputvrml",      0, 0, nullptr);
+  const auto OUTPUTKML       = cdo_operator_add("outputkml",       0, 0, nullptr);
   // clang-format on
 
-  const auto operatorID = cdoOperatorID();
+  const auto operatorID = cdo_operator_id();
+
+  const auto printHeader = (operatorID != OUTPUTTRI && operatorID != OUTPUTKML);
 
   if (operatorID == OUTPUTVECTOR)
     {
-      operatorInputArg("increment");
-      operatorCheckArgc(1);
-      ninc = parameter2int(cdoOperatorArgv(0));
-      if (ninc < 1) cdoAbort("Increment must be greater than 0!");
+      operator_input_arg("increment");
+      operator_check_argc(1);
+      ninc = parameter_to_int(cdo_operator_argv(0));
+      if (ninc < 1) cdo_abort("Increment must be greater than 0!");
     }
 
-  if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT) luse_grid_corner = true;
+  const auto needCellCorners = (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT || operatorID == OUTPUTKML);
 
-  if (operatorID == OUTPUTCENTERCPT || operatorID == OUTPUTBOUNDSCPT || operatorID == OUTPUTVRML)
+  if (operatorID == OUTPUTCENTERCPT || operatorID == OUTPUTBOUNDSCPT || operatorID == OUTPUTKML || operatorID == OUTPUTVRML)
     {
-      operatorCheckArgc(1);
-      auto cpt_file = cdoOperatorArgv(0).c_str();
+      operator_check_argc(1);
+      auto cpt_file = cdo_operator_argv(0).c_str();
 
       auto cpt_fp = fopen(cpt_file, "r");
-      if (cpt_fp == nullptr) cdoAbort("Open failed on color palette table %s", cpt_file);
+      if (cpt_fp == nullptr) cdo_abort("Open failed on color palette table %s", cpt_file);
 
-      const auto status = cptRead(cpt_fp, &cpt);
-      if (status != 0) cdoAbort("Error during read of color palette table %s", cpt_file);
+      const auto status = cpt_read(cpt_fp, &cpt);
+      if (status != 0) cdo_abort("Error during read of color palette table %s", cpt_file);
 
-      if (Options::cdoVerbose) cptWrite(stderr, cpt);
+      if (Options::cdoVerbose) cpt_write(stderr, cpt);
     }
 
-  const auto streamID = cdoOpenRead(0);
+  const auto streamID = cdo_open_read(0);
 
-  const auto vlistID = cdoStreamInqVlist(streamID);
+  const auto vlistID = cdo_stream_inq_vlist(streamID);
   const auto taxisID = vlistInqTaxis(vlistID);
 
   int varID = 0;
@@ -333,7 +426,7 @@ Outputgmt(void *process)
 
   gridID = generate_full_cell_grid(gridID);
 
-  if (!gridHasCoordinates(gridID)) cdoAbort("Cell center coordinates missing!");
+  if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
 
   const auto gridsize = gridInqSize(gridID);
 
@@ -358,19 +451,19 @@ Outputgmt(void *process)
       nlat = 1;
     }
 
-  if (Options::cdoVerbose && lhov) cdoPrint("Process hovmoeller data");
-  if (Options::cdoVerbose && lzon) cdoPrint("Process zonal data");
-  if (Options::cdoVerbose && lmer) cdoPrint("Process meridional data");
+  if (Options::cdoVerbose && lhov) cdo_print("Process hovmoeller data");
+  if (Options::cdoVerbose && lzon) cdo_print("Process zonal data");
+  if (Options::cdoVerbose && lmer) cdo_print("Process meridional data");
   /*
   if ( lzon || lmer )
     {
       if ( operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT )
-        cdoAbort("Bounds not available for zonal/meridional data!");
+        cdo_abort("Bounds not available for zonal/meridional data!");
     }
   */
   if (lhov)
     {
-      if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT) cdoAbort("Bounds not available hovmoeller data!");
+      if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT) cdo_abort("Bounds not available hovmoeller data!");
     }
 
   const int ncorner = (gridInqType(gridID) == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
@@ -412,16 +505,16 @@ Outputgmt(void *process)
 
   Varray<double> zaxis_center_lev(nlev), zaxis_lower_lev(nlev), zaxis_upper_lev(nlev);
 
-  cdoZaxisInqLevels(zaxisID, zaxis_center_lev.data());
+  cdo_zaxis_inq_levels(zaxisID, zaxis_center_lev.data());
 
-  if (luse_grid_corner)
+  if (needCellCorners)
     {
-      if (ncorner == 0) cdoAbort("Number of cell corners undefined!");
+      if (ncorner == 0) cdo_abort("Number of cell corners undefined!");
       const size_t nalloc = ncorner * gridsize;
       grid_corner_lat.resize(nalloc);
       grid_corner_lon.resize(nalloc);
 
-      if (!gridHasBounds(gridID)) cdoAbort("Cell corner coordinates missing!");
+      if (!gridHasBounds(gridID)) cdo_abort("Cell corner coordinates missing!");
 
       gridInqYbounds(gridID, grid_corner_lat.data());
       gridInqXbounds(gridID, grid_corner_lon.data());
@@ -459,43 +552,38 @@ Outputgmt(void *process)
     }
 
   Varray<double> uf, vf;
-  if (operatorID == OUTPUTVECTOR)
-    {
-      uf.resize(gridsize);
-      vf.resize(gridsize);
-    }
+  if (operatorID == OUTPUTVECTOR) uf.resize(gridsize);
+  if (operatorID == OUTPUTVECTOR) vf.resize(gridsize);
 
   int tsID = 0;
   while (true)
     {
-      const auto nrecs = cdoStreamInqTimestep(streamID, tsID);
+      const auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
       if (nrecs == 0) break;
 
       const auto vdate = taxisInqVdate(taxisID);
       const auto vtime = taxisInqVtime(taxisID);
-      const auto vdateString = dateToString(vdate);
-      const auto vtimeString = timeToString(vtime);
+      const auto vdateString = date_to_string(vdate);
+      const auto vtimeString = time_to_string(vtime);
 
-      if (tsID == 0 && operatorID != OUTPUTTRI)
+      if (tsID == 0 && printHeader)
         {
           if (operatorID == OUTPUTVRML) printf("#VRML V2.0 utf8\n\n");
 #ifdef VERSION
           fprintf(stdout, "# Generated by CDO version %s\n", VERSION);
           fprintf(stdout, "#\n");
 #endif
-          fprintf(stdout, "# Operator = %s\n", cdoOperatorName(operatorID));
-          if (lhov)
-            fprintf(stdout, "# Mode     = hovmoeller\n");
-          else if (lzon)
-            fprintf(stdout, "# Mode     = zonal\n");
-          else if (lmer)
-            fprintf(stdout, "# Mode     = meridional\n");
-          else
-            fprintf(stdout, "# Mode     = horizonal\n");
+          fprintf(stdout, "# Operator = %s\n", cdo_operator_name(operatorID));
+          // clang-format off
+          if      (lhov) fprintf(stdout, "# Mode     = hovmoeller\n");
+          else if (lzon) fprintf(stdout, "# Mode     = zonal\n");
+          else if (lmer) fprintf(stdout, "# Mode     = meridional\n");
+          else           fprintf(stdout, "# Mode     = horizonal\n");
+          // clang-format on
 
           if (operatorID == OUTPUTVECTOR) fprintf(stdout, "# Increment = %d\n", ninc);
           fprintf(stdout, "#\n");
-          fprintf(stdout, "# Stream = %s\n", cdoGetStreamName(0));
+          fprintf(stdout, "# Stream = %s\n", cdo_get_stream_name(0));
           fprintf(stdout, "# Date   = %s\n", vdateString.c_str());
           fprintf(stdout, "# Time   = %s\n", vtimeString.c_str());
           fprintf(stdout, "# Name   = %s\n", varname);
@@ -507,27 +595,27 @@ Outputgmt(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           int levelID;
-          cdoInqRecord(streamID, &varID, &levelID);
+          cdo_inq_record(streamID, &varID, &levelID);
 
           if (varID != varID0) continue;
           if (recID > 0 && !lzon && !lmer) continue;
 
-          cdoReadRecord(streamID, array.data(), &nmiss);
+          cdo_read_record(streamID, array.data(), &nmiss);
 
           if (operatorID == OUTPUTCENTER2 && grid_is_circular) make_cyclic(array.data(), array2.data(), nlon, nlat);
 
           const auto level = zaxis_center_lev[levelID];
 
-          if ((tsID == 0 || lzon || lmer) && operatorID != OUTPUTTRI) fprintf(stdout, "# Level  = %g\n", level);
+          if ((tsID == 0 || lzon || lmer) && printHeader) fprintf(stdout, "# Level  = %g\n", level);
           if (lhov) fprintf(stdout, "# Timestep = %d\n", tsID + 1);
 
-          if (operatorID != OUTPUTTRI) fprintf(stdout, "#\n");
+          if (printHeader) fprintf(stdout, "#\n");
 
           if (operatorID == OUTPUTCENTER || operatorID == OUTPUTCENTER2 || operatorID == OUTPUTCENTERCPT)
             {
               if (Options::cdoVerbose)
                 {
-                  const auto mm = varrayMinMaxMV(gridsize, array, missval);
+                  const auto mm = varray_min_max_mv(gridsize, array, missval);
                   const auto range = mm.max - mm.min;
                   fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", mm.min, mm.max, range / 20);
                   fprintf(stderr, "pscontour -K -JQ0/10i -Rd -I -Cgmt.cpt data.gmt > gmtplot.ps\n");
@@ -567,7 +655,7 @@ Outputgmt(void *process)
             }
           else if (operatorID == OUTPUTTRI)
             {
-              if (gridInqType(gridID) != GRID_CURVILINEAR) cdoAbort("Unsupported grid!");
+              if (gridInqType(gridID) != GRID_CURVILINEAR) cdo_abort("Unsupported grid!");
 
               const long mlon = nlon - 1;
               // if ( gridIsCircular(gridID) ) mlon = nlon;
@@ -586,13 +674,17 @@ Outputgmt(void *process)
                     fprintf(stdout, "%d   %d   %d\n", c1, c2, c3);
                   }
             }
+          else if (operatorID == OUTPUTKML)
+            {
+              output_kml(gridsize, array, ncorner, grid_corner_lat, grid_corner_lon, grid_mask, missval, cpt);
+            }
           else if (operatorID == OUTPUTVECTOR)
             {
-              if (nrecs < 2) cdoAbort("Too few fields!");
+              if (nrecs < 2) cdo_abort("Too few fields!");
 
-              varrayCopy(gridsize, array, uf);
-              cdoInqRecord(streamID, &varID, &levelID);
-              cdoReadRecord(streamID, vf.data(), &nmiss);
+              varray_copy(gridsize, array, uf);
+              cdo_inq_record(streamID, &varID, &levelID);
+              cdo_read_record(streamID, vf.data(), &nmiss);
 
               output_vector(nlon, nlat, ninc, grid_center_lon, grid_center_lat, uf, vf);
 
@@ -606,7 +698,7 @@ Outputgmt(void *process)
             {
               if (Options::cdoVerbose)
                 {
-                  const auto mm = varrayMinMaxMV(gridsize, array, missval);
+                  const auto mm = varray_min_max_mv(gridsize, array, missval);
                   const auto range = mm.max - mm.min;
                   fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", mm.min, mm.max, range / 20);
                   fprintf(stderr, "psxy -K -JQ0/10i -Rd -L -Cgmt.cpt -m data.gmt > gmtplot.ps\n");
@@ -628,7 +720,8 @@ Outputgmt(void *process)
 
                   if (operatorID == OUTPUTBOUNDSCPT)
                     {
-                      output_boundscpt(array[i], missval, cpt);
+                      auto rgb = get_rgb(array[i], missval, cpt);
+                      fprintf(stdout, " -G%d/%d/%d", rgb[0], rgb[1], rgb[2]);
                     }
 
                   fprintf(stdout, "\n");
@@ -643,7 +736,7 @@ Outputgmt(void *process)
                     }
                   else if (lhov)
                     {
-                      cdoAbort("Implementation for hovmoeller data missing!");
+                      cdo_abort("Implementation for hovmoeller data missing!");
                     }
                   else
                     {
@@ -664,9 +757,9 @@ Outputgmt(void *process)
       tsID++;
     }
 
-  cdoStreamClose(streamID);
+  cdo_stream_close(streamID);
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
