@@ -50,6 +50,8 @@ Recttocomplex(void *process)
     {
       const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
       if (nrecs == 0) break;
+      const auto nrecs2 = cdo_stream_inq_timestep(streamID2, tsID);
+      if (nrecs2 == 0) break;
 
       cdo_taxis_copy_timestep(taxisID3, taxisID1);
       cdo_def_timestep(streamID3, tsID);
