@@ -1,18 +1,9 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2021 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
+          Oliver Heidmann
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 #ifdef HAVE_CONFIG_H
@@ -336,7 +327,7 @@ cdo_get_stream_name(int p_streamIndex)
   return streamName;
 }
 
-const char *
+std::string
 cdo_get_command_from_in_stream(int p_streamIndex)
 {
   const std::shared_ptr<Process> process = cdoGetInputChild(p_streamIndex);
@@ -344,7 +335,7 @@ cdo_get_command_from_in_stream(int p_streamIndex)
     {
       return cdo_get_stream_name(p_streamIndex);
     }
-  return process->get_command();
+  return process->operatorName;
 }
 
 bool
@@ -353,10 +344,10 @@ cdo_assert_files_only()
   return localProcess->has_no_pipes();
 }
 
-const char *
+std::string
 cdo_get_obase()
 {
-  return localProcess->m_obase;
+  return localProcess->get_obase();
 }
 
 void
@@ -369,7 +360,7 @@ cdo_initialize(void *p_process)
 #ifdef HAVE_LIBPTHREAD
   localProcess->threadID = pthread_self();
 #endif
-  Debug(PROCESS, "Initializing process: %s", localProcess->m_operatorCommand);
+  Debug(PROCESS, "Initializing process: %s (id: %d)", localProcess->operatorName , localProcess->get_id());
 
 #ifdef HAVE_LIBPTHREAD
   Debug(PROCESS, "process %d thread %ld", localProcess->m_ID, pthread_self());
