@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2021 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 /*
   This is a C library of the Fortran SCRIP version 1.4
@@ -84,10 +74,10 @@ void
 remap_set_int(int remapvar, int value)
 {
   // clang-format off
-  if      (remapvar == REMAP_WRITE_REMAP)   remap_write_remap = value > 0;
+  if      (remapvar == REMAP_WRITE_REMAP)   remap_write_remap = (value > 0);
   else if (remapvar == REMAP_MAX_ITER)      remap_max_iter = value;
   else if (remapvar == REMAP_NUM_SRCH_BINS) remap_num_srch_bins = value;
-  else if (remapvar == REMAP_GENWEIGHTS)    remap_gen_weights = value > 0;
+  else if (remapvar == REMAP_GENWEIGHTS)    remap_gen_weights = (value > 0);
   else    cdo_abort("Unsupported remap variable (%d)!", remapvar);
   // clang-format on
 }
@@ -97,7 +87,7 @@ static void
 boundboxFromCorners(size_t size, size_t nc, const Varray<double> &corner_lon, const Varray<double> &corner_lat, float *bound_box)
 {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(bound_box, corner_lat, corner_lon, nc, size)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < size; ++i)
     {
@@ -126,7 +116,7 @@ boundboxFromCenter(bool lonIsCyclic, size_t size, size_t nx, size_t ny, const Va
                    const Varray<double> &center_lat, float *bound_box)
 {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(lonIsCyclic, size, nx, ny, center_lon, center_lat, bound_box)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t n = 0; n < size; n++)
     {
@@ -201,7 +191,7 @@ check_lon_range(const char *txt, size_t nlons, Varray<double> &lons)
       double minval = 1.e36;
       double maxval = -1.e36;
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlons, lons) reduction(min : minval) reduction(max : maxval)
+#pragma omp parallel for default(shared) schedule(static) reduction(min : minval) reduction(max : maxval)
 #endif
       for (size_t i = 0; i < nlons; ++i)
         {
@@ -213,7 +203,7 @@ check_lon_range(const char *txt, size_t nlons, Varray<double> &lons)
     }
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlons, lons)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < nlons; ++i)
     {
@@ -236,7 +226,7 @@ check_lat_range(const char *txt, size_t nlats, Varray<double> &lats)
       double minval = 1.e36;
       double maxval = -1.e36;
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlats, lats) reduction(min : minval) reduction(max : maxval)
+#pragma omp parallel for default(shared) schedule(static) reduction(min : minval) reduction(max : maxval)
 #endif
       for (size_t i = 0; i < nlats; ++i)
         {
@@ -248,7 +238,7 @@ check_lat_range(const char *txt, size_t nlats, Varray<double> &lats)
     }
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlats, lats)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < nlats; ++i)
     {
@@ -263,12 +253,12 @@ check_lon_boundbox_range(size_t nlons, float *bound_box)
   assert(bound_box != nullptr);
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlons, bound_box)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t n = 0; n < nlons; ++n)
     {
       const auto n4 = n << 2;
-      if (fabsf(bound_box[n4 + 3] - bound_box[n4 + 2]) > PI_f)
+      if (std::fabs(bound_box[n4 + 3] - bound_box[n4 + 2]) > PI_f)
         {
           bound_box[n4 + 2] = 0.0f;
           bound_box[n4 + 3] = PI2_f;
@@ -282,7 +272,7 @@ check_lat_boundbox_range(size_t nlats, float *bound_box, Varray<double> &lats)
   assert(bound_box != nullptr);
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlats, bound_box, lats)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t n = 0; n < nlats; ++n)
     {
@@ -346,7 +336,7 @@ convert_bounds_reg2d(size_t n, const Varray<double> &boundsIn, Varray<double> &b
 }
 
 static void
-remap_define_reg2d(int gridID, RemapGrid &grid)
+remap_define_reg2d(int gridID, RemapGrid &grid, bool conservMapping, const char *txt)
 {
   const auto nx = grid.dims[0];
   const auto ny = grid.dims[1];
@@ -388,6 +378,7 @@ remap_define_reg2d(int gridID, RemapGrid &grid)
     }
   else
     {
+      if (conservMapping && nx == 1) cdo_abort("Longitude bounds of %s grid missing!", txt);
       grid_gen_corners(nx, grid.reg2d_center_lon.data(), grid.reg2d_corner_lon.data());
     }
 
@@ -400,6 +391,7 @@ remap_define_reg2d(int gridID, RemapGrid &grid)
     }
   else
     {
+      if (conservMapping && ny == 1) cdo_abort("Latitude bounds of %s grid missing!", txt);
       grid_gen_corners(ny, grid.reg2d_center_lat.data(), grid.reg2d_corner_lat.data());
       grid_check_lat_borders_rad(ny + 1, grid.reg2d_corner_lat);
     }
@@ -408,7 +400,7 @@ remap_define_reg2d(int gridID, RemapGrid &grid)
 static void
 remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *txt)
 {
-  bool lgrid_destroy = false;
+  bool destroyGrid = false;
   int gridID_gme = -1;
 
   const auto gridtype = gridInqType(grid.gridID);
@@ -424,12 +416,12 @@ remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *tx
         }
       else if (gridtype == GRID_GAUSSIAN_REDUCED)
         {
-          lgrid_destroy = true;
+          destroyGrid = true;
           gridID = gridToUnstructured(grid.gridID, 1);
         }
       else if (remap_write_remap || grid.type != RemapGridType::Reg2D)
         {
-          lgrid_destroy = true;
+          destroyGrid = true;
           gridID = gridToCurvilinear(grid.gridID, 1);
         }
     }
@@ -455,7 +447,7 @@ remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *tx
   // Initialize logical mask
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(gridsize, grid)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < gridsize; ++i) grid.mask[i] = 1;
 
@@ -493,7 +485,7 @@ remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *tx
       cdo_grid_to_radian(gridID, CDI_YAXIS, grid.num_cell_corners * grid.size, grid.cell_corner_lat.data(), "grid corner lat");
     }
 
-  if (lgrid_destroy) gridDestroy(gridID);
+  if (destroyGrid) gridDestroy(gridID);
 
   // Convert longitudes to 0,2pi interval
   check_lon_range(txt, grid.size, grid.cell_center_lon);
@@ -561,10 +553,8 @@ remap_grid_alloc(RemapMethod mapType, RemapGrid &grid)
       grid.cell_center_lat.resize(grid.size);
     }
 
-  if (mapType == RemapMethod::CONSERV_SCRIP || mapType == RemapMethod::CONSERV)
-    {
-      grid.cell_area.resize(grid.size, 0.0);
-    }
+  const auto needCellarea = (mapType == RemapMethod::CONSERV_SCRIP || mapType == RemapMethod::CONSERV);
+  if (needCellarea) grid.cell_area.resize(grid.size, 0.0);
 
   grid.cell_frac.resize(grid.size, 0.0);
 
@@ -611,6 +601,22 @@ remap_grid_free(RemapGrid &grid)
   if (grid.tmpgridID != -1) gridDestroy(grid.tmpgridID);
 }
 
+static void
+check_for_convex_cells(RemapGrid &tgt_grid)
+{
+  if (tgt_grid.type == RemapGridType::Reg2D) return;
+
+  const auto numCellCorners = tgt_grid.num_cell_corners;
+  if (numCellCorners <= 4) return;
+
+  auto numCells = tgt_grid.size;
+  if (numCells > 1000) numCells = 1000;
+
+  for (size_t i = 0; i < numCells; i++)
+    {
+    }
+}
+
 void
 remap_search_init(RemapMethod mapType, RemapSearch &search, RemapGrid &src_grid, RemapGrid &tgt_grid)
 {
@@ -636,11 +642,13 @@ remap_search_init(RemapMethod mapType, RemapSearch &search, RemapGrid &src_grid,
   auto useCellsearch = (mapType == RemapMethod::CONSERV)
                        && (cellSearchMethod == CellSearchMethod::spherepart || src_grid.type == RemapGridType::Reg2D);
 
-  auto start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
+  const char *searchMethodStr = NULL;
+  const auto start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
 
   if (usePointsearch)
     {
-      bool xIsCyclic = src_grid.is_cyclic;
+      searchMethodStr = "Point search";
+      const auto xIsCyclic = src_grid.is_cyclic;
       if (src_grid.type == RemapGridType::Reg2D)
         grid_point_search_create_reg_2d(search.gps, xIsCyclic, src_grid.dims, src_grid.reg2d_center_lon, src_grid.reg2d_center_lat);
       else
@@ -648,21 +656,21 @@ remap_search_init(RemapMethod mapType, RemapSearch &search, RemapGrid &src_grid,
                                  src_grid.cell_center_lat);
 
       if (src_grid.lextrapolate) grid_point_search_extrapolate(search.gps);
-
-      if (Options::cdoVerbose) cdo_print("Point search created: %.2f seconds", cdo_get_wtime() - start);
     }
   else if (useCellsearch)
     {
+      searchMethodStr = "Cell search";
       if (src_grid.type == RemapGridType::Reg2D)
         grid_cell_search_create_reg_2d(search.gcs, src_grid.dims, src_grid.reg2d_corner_lon, src_grid.reg2d_corner_lat);
       else
         grid_cell_search_create(search.gcs, src_grid.size, src_grid.num_cell_corners, src_grid.cell_corner_lon,
                                 src_grid.cell_corner_lat);
 
-      if (Options::cdoVerbose) cdo_print("Cell search created: %.2f seconds", cdo_get_wtime() - start);
+      // check_for_convex_cells(tgt_grid);
     }
   else if (!(src_grid.type == RemapGridType::Reg2D || tgt_grid.type == RemapGridType::Reg2D))
     {
+      searchMethodStr = "Latitude bins";
       search.srcBins.cell_bound_box.resize(4 * src_grid.size);
       if (tgt_grid.luse_cell_corners) search.tgtBins.cell_bound_box.resize(4 * tgt_grid.size);
 
@@ -677,9 +685,9 @@ remap_search_init(RemapMethod mapType, RemapSearch &search, RemapGrid &src_grid,
           varray_free(search.srcBins.bin_lats);
           if (mapType == RemapMethod::CONSERV) varray_free(search.tgtBins.cell_bound_box);
         }
-
-      if (Options::cdoVerbose) cdo_print("Latitude bins created: %.2f seconds", cdo_get_wtime() - start);
     }
+
+  if (Options::cdoVerbose && searchMethodStr) cdo_print("%s created: %.2f seconds", searchMethodStr, cdo_get_wtime() - start);
 }
 
 void
@@ -766,8 +774,8 @@ remap_init_grids(RemapMethod mapType, bool lextrapolate, int gridID1, RemapGrid
   const auto sgridID = src_grid.gridID;
   if (gridInqSize(sgridID) > 1 && gridProjIsSupported(sgridID))
     {
-      bool lbounds = true;
-      src_grid.gridID = gridID1 = gridToCurvilinear(src_grid.gridID, lbounds);
+      const bool withBounds = true;
+      src_grid.gridID = gridID1 = gridToCurvilinear(src_grid.gridID, withBounds);
       src_grid.tmpgridID = src_grid.gridID;
     }
 
@@ -777,8 +785,9 @@ remap_init_grids(RemapMethod mapType, bool lextrapolate, int gridID1, RemapGrid
 
   if (src_grid.type == RemapGridType::Reg2D || tgt_grid.type == RemapGridType::Reg2D)
     {
-      if (src_grid.type == RemapGridType::Reg2D) remap_define_reg2d(reg2d_src_gridID, src_grid);
-      if (tgt_grid.type == RemapGridType::Reg2D) remap_define_reg2d(reg2d_tgt_gridID, tgt_grid);
+      const auto conservMapping = (mapType == RemapMethod::CONSERV);
+      if (src_grid.type == RemapGridType::Reg2D) remap_define_reg2d(reg2d_src_gridID, src_grid, conservMapping, "source");
+      if (tgt_grid.type == RemapGridType::Reg2D) remap_define_reg2d(reg2d_tgt_gridID, tgt_grid, conservMapping, "target");
     }
 }
 
@@ -833,7 +842,7 @@ remap_stat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &
 #ifdef HAVE_OPENMP4
 #pragma omp simd
 #endif
-  for (size_t n = 0; n < rv.num_links; ++n) tgt_count[rv.tgt_cell_add[n]]++;
+  for (size_t n = 0; n < rv.num_links; ++n) tgt_count[rv.tgtCellIndices[n]]++;
 
   size_t imin = SIZE_MAX;
   size_t imax = 0;
@@ -897,7 +906,7 @@ remap_gradients(RemapGrid &grid, const Varray<short> &mask, const Varray<T> &arr
   auto ny = grid.dims[1];
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(grid_size, gradients, grid, nx, ny, array, mask)
+#pragma omp parallel for default(shared)
 #endif
   for (size_t n = 0; n < grid_size; ++n)
     {
@@ -1063,7 +1072,7 @@ remap_gradients(RemapGrid &grid, const Field &field, RemapGradients &gradients)
   auto grid_size = grid.size;
   Varray<short> mask(grid_size);
 #ifdef _OPENMP
-#pragma omp parallel for default(none) schedule(static) shared(grid_size, grid, mask)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < grid_size; ++i) mask[i] = (grid.mask[i] > 0);
 
